#!/bin/bash

# ------------------------------------------------------------
# download and install the initial VM for 
#
#       OpenSecurity V0.1
#
# This has been originally a Windows only BAT file.
#
# ... but coding this makes your head hurt and
# supporting this "technology" any further by adding
# software to the world relying on CMD.exe is an act
# against humanity and should be punished by jail.
#
# To be called with the OpenSecurity installation folder
# like C:\Program Files\OpenSecurity as first argument
#
#
# (C)opyright 2014, AIT Austrian Instiitute of Technology
# ------------------------------------------------------------


# ------------------------------------------------------------
# vars

OPENSECURITY_DIR="${1}"
OPENSECURITY_PORT=8080
OVA_TEMPLATE_URL="http://service.x-net.at/opensecurity/OsecVM_latest.ova"


# ------------------------------------------------------------
# code


# ------------------------------
# press a key message to the user
# 
function press_a_key() {
    echo "press a key ..."
    read -n 1 X
    echo
}


# ------------------------------
# read a value from the windows regsitry
#
#   $1  ...     full registry key
#   $2  ...     value name
#   stdout      the value found
# 
function reg_read() {

    test -z "${1}" && return
    
    if [ -z "${2}" ]; then
        reg.exe query "${1}"
    else
        reg.exe query "${1}" /v "${2}" | grep "${2}" | sed 's/^.*REG_[A-Z]* *//' 
    fi 
}


# ------------------------------
# turns a windows path into a cygwin path
#
#   $1  ...     windows path
#   stdout      the value found
#
function sanitize_path() {
    test -z "${1}" && return
    echo $(cygpath -u "${1}") 
}


# ------------------------------
# main ...
#

# check opensecurity folder
#
OPENSECURITY_DIR=$(sanitize_path "${OPENSECURITY_DIR}")
OSECVM_IMAGE="${OPENSECURITY_DIR}/install/OsecVM.ova"
if [ ! -d "${OPENSECURITY_DIR}" ]; then
    echo "warning: no valid installation folder specified"
fi

# look up VirtulBox installation
#
VBOX_MANAGER="$(reg_read 'HKLM\Software\Oracle\VirtualBox' 'InstallDir')VBoxManage.exe"
VBOX_MANAGER=$(sanitize_path "${VBOX_MANAGER}")
if [ ! -x "${VBOX_MANAGER}" ]; then
    echo "can't execute VBoxManage.exe - is VirtualBox installed?"
    echo "looked at: "$(cygpath -w ${VBOX_MANAGER})""
    press_a_key
    exit 1
fi

# enforce VirtualBox to "feel good" by calling a function
# (that is to "warm up" VirtualBox DCOM server ...)
#
VBOX_VERSION=$("${VBOX_MANAGER}" -version)
"${VBOX_MANAGER}" list vms &> /dev/null

# download OSec.VM
#
if [ ! -e "${OSECVM_IMAGE}" ]; then
    
    echo "checking for local OsecVM.ova image instance ..."

    # we would like to place the OsecVM.ova image
    # inside the installation folder:
    #
    #       C:\Program Files\OpenSecurity
    #
    # BUT: this script also works in deep with
    #      VirtualBox and this one has a very
    #      picky understanding of user/admin
    #      permissions on Windows
    #
    # So, in oder to place any files under
    # C:\Program Files\OpenSecurity we need
    # this script running with elevated
    # user rights.
    #
    # ... which make VirtualBox managed VMs
    # by the install user impossible, since
    # VirtualBox gets confused by the User
    # privileges setup here in Windows
    # (Well, who doesn't anyway?).

    # user space place of OsecVM.ova image
    # ---> USER/AppData/Roaming/OpenSecurity
   
    mkdir -p $(cygpath -O)/../AppData/Roaming/OpenSecurity &> /dev/null
    if [ ! "${?}" = "0" ]; then
        echo "failed to create folder $(cygpath -O)/../AppData/Roaming/OpenSecurity"
        echo "unable to proceed ..."
        press_a_key
        exit 1
    fi

    OSECVM_IMAGE=$(cygpath -O)/../AppData/Roaming/OpenSecurity/OsecVM.ova
    if [ ! -e "${OSECVM_IMAGE}" ]; then
        echo "downloading OSecVM.ova image ..."
        wget -O "${OSECVM_IMAGE}" "${OVA_TEMPLATE_URL}" 
        if [ "${?}" != "0" ]; then
            echo "failed to download OsecVM.ova"
            press_a_key
            exit 1
        fi    
    fi
fi
echo "using $(cygpath -w ${OSECVM_IMAGE}) as inital VM image to import ..."


# the next operations to produce some file I/O
# (especially SecurityDVM/SecurityDVM,vmdk)
# we therefore switch into the user's home
pushd  $(cygpath -O)/.. &> /dev/null

# VirtualBox works on the User's home ...
# which is different in cygwin and pure windows
# yet another idiosyncrasy ... -.-
export HOME="$(cygpath -w "$(cygpath -O)/..")"
echo "HOME: ${HOME}"

VDISK_IMAGE="VirtualBox VMs\SecurityDVM\SecurityDVM.vmdk"

# import VM 
#
"${VBOX_MANAGER}" list vms | grep SecurityDVM &> /dev/null
if [ ! "${?}" = "0" ]; then
    echo "importing VM: ${OSECVM_IMAGE}"
    "${VBOX_MANAGER}" import "$(cygpath -w "${OSECVM_IMAGE}")" --vsys 0 --vmname SecurityDVM --unit 12 --disk "${VDISK_IMAGE}"
else
    echo "found SecurityDVM already present in VBox reusing it."
    echo "if you want a complete new import please remove the VM first."
fi

# grab VM storage controller and port 
#
VDISK_SETUP=$("${VBOX_MANAGER}" showvminfo SecurityDVM | grep SecurityDVM.vmdk | cut -d ':' -f 1 | tr '(),' '   ')
VDISK_CONTROLLER=$(echo ${VDISK_SETUP} | gawk '{print $1;}')
VDISK_PORT=$(echo ${VDISK_SETUP} | gawk '{print $2;}')
VDISK_DEVICE=$(echo ${VDISK_SETUP} | gawk '{print $3;}')
if [ -z "${VDISK_CONTROLLER}" ]; then
    echo "unable to grab virtual disk controller in VM."
    echo "this shouldn't happen. It's a bug."
    press_a_key
    popd
    exit 1
fi

# detach disk image
#
echo "detaching disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --medium none

# turn disk image into writethrough
#
echo "turning disk image into writetrough ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --device ${VDISK_DEVICE} --type hdd --mtype writethrough --medium "${VDISK_IMAGE}" 

# detach disk image
#
echo "detaching disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --medium none

# immutablize disk
#
echo "reattach immutable disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --device ${VDISK_DEVICE} --type hdd --mtype immutable --medium "${VDISK_IMAGE}"

# return to current working dir
popd &> /dev/null

echo "imported initial OsecVM.ova image"
